@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+from imio.email.dms import dev_mode
+from imio.email.dms.utils import reception_date
 from imio.email.parser.parser import Parser  # noqa
 from mailparser import MailParser
 
@@ -68,13 +70,14 @@ class IMAPEmailHandler(object):
         """Fetch all waiting messages"""
         args = [u"NOT KEYWORD imported", u"NOT KEYWORD unsupported", u"NOT KEYWORD error", u"NOT KEYWORD ignored"]
         # args = [u"KEYWORD imported"]
+        # args = ['SUBJECT "PERMANNE"']
         res, data = self.connection.search(None, *args)
         if res != "OK":
             logger.error("Unable to fetch mails")
             return []
         waiting = []
         for mail_id in data[0].split():
-            # if mail_id != b'3335': continue
+            # if mail_id != b'388': continue
             mail = self.get_mail(mail_id)
             if not mail:
                 continue
@@ -94,7 +97,10 @@ class IMAPEmailHandler(object):
 
     def list_last_emails(self, nb=20):
         """List last messages"""
-        res, data = self.connection.search(None, 'ALL')
+        # args = [u"NOT KEYWORD imported", u"NOT KEYWORD unsupported", u"NOT KEYWORD error", u"NOT KEYWORD ignored"]
+        # args = ['SUBJECT "PERMANNE"']
+        args = ['ALL']
+        res, data = self.connection.search(None, *args)
         if res != "OK":
             logger.error("Unable to fetch mails")
             return []
@@ -108,13 +114,16 @@ class IMAPEmailHandler(object):
             mail = self.get_mail(mail_id)
             if not mail:
                 continue
-            parser = Parser(mail)
+            parser = Parser(mail, dev_mode, mail_id)
             parsed_orig_mail = MailParser(mail)
+            r_date = reception_date(mail)
             if isinstance(mail_id, bytes):
                 mail_id = mail_id.decode()
                 flags = [fl.decode() for fl in flags]
-            lst.append(u"{}: '{}', '{}', {}, ''{}".format(mail_id, parsed_orig_mail.subject, parser.headers['Subject'],
-                                                          parsed_orig_mail.headers['From'], flags))
+            lst.append(u"{}, {}: '{}', '{}', '{}', '{}', {}".format(r_date, mail_id, parsed_orig_mail.headers['From'],
+                       parsed_orig_mail.subject, parser.parsed_message.headers['From'], parser.headers['Subject'],
+                       flags))
+                       # parser.message.get('message-id'), flags))
             logger.info(lst[-1])
         return lst
 
