@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Usage: process_mails FILE [--requeue_errors] [--list_emails=<number>] [--get_eml=<mail_id>]  [--gen_pdf=<mail_id>]
+Usage: process_mails FILE [--requeue_errors] [--list_emails=<number>] [--get_eml=<mail_id>] [--gen_pdf=<mail_id>]
                           [--get_eml_orig] [--stats]
 
 Arguments:
@@ -23,16 +23,23 @@ from email.mime.base import MIMEBase
 from email.mime.multipart import MIMEMultipart
 from email.mime.text import MIMEText
 from hashlib import md5
+from imio.email.dms import dev_mode
+from imio.email.dms import logger
 from imio.email.dms.imap import IMAPEmailHandler
 from imio.email.dms.imap import MailData
+from imio.email.dms.utils import get_next_id
+from imio.email.dms.utils import get_reduced_size
 from imio.email.dms.utils import safe_unicode
+from imio.email.dms.utils import save_as_eml
+from imio.email.dms.utils import set_next_id
 from imio.email.parser.parser import Parser  # noqa
 from io import BytesIO
+from PIL import Image
+from PIL import UnidentifiedImageError
 from smtplib import SMTP
 import configparser
 import imaplib
 import json
-import logging
 import os
 import re
 import requests
@@ -40,19 +47,15 @@ import six
 import sys
 import tarfile
 import zc.lockfile
-from imio.email.dms.utils import save_as_eml
 
 try:
     from pathlib import Path
 except ImportError:
-    from pathlib2 import Path
+    from pathlib2 import Path  # noqa
 
-logger = logging.getLogger("imio.email.dms")
-logger.setLevel(logging.INFO)
-chandler = logging.StreamHandler()
-chandler.setLevel(logging.INFO)
-logger.addHandler(chandler)
 
+dev_infos = {'nid': None}
+img_size_limit = 1024
 
 ERROR_MAIL = u"""
 Problematic mail is attached.\n
@@ -109,7 +112,7 @@ Le mail concerné est en pièce jointe.\n
 Client ID : {0}
 IMAP login : {1}
 mail id : {2}
-pattern : "{4}"
+pattern : "caché"
 """
 
 RESULT_MAIL = u"""
@@ -151,13 +154,12 @@ def notify_exception(config, mail_id, mail, error):
     msg.attach(main_text)
 
     attachment = MIMEBase("message", "rfc822")
-    attachment.set_payload(mail.as_string())
+    attachment.set_payload(mail.as_string(), 'utf8')
     attachment.add_header("Content-Disposition", "inline")
     msg.attach(attachment)
 
     smtp = SMTP(str(smtp_infos["host"]), int(smtp_infos["port"]))
-    msg_content = msg.as_string().encode("utf8") if six.PY3 else msg.as_string()
-    smtp.sendmail(sender, recipient, msg_content)
+    smtp.send_message(msg)
     smtp.quit()
 
 
@@ -175,13 +177,12 @@ def notify_unsupported_origin(config, mail, from_):
     msg.attach(main_text)
 
     attachment = MIMEBase("message", "rfc822")
-    attachment.set_payload(mail.as_string())
+    attachment.set_payload(mail.as_string(), 'utf8')
     attachment.add_header("Content-Disposition", "inline")
     msg.attach(attachment)
 
     smtp = SMTP(str(smtp_infos["host"]), int(smtp_infos["port"]))
-    msg_content = msg.as_string().encode("utf8") if six.PY3 else msg.as_string()
-    smtp.sendmail(sender, from_, msg_content)
+    smtp.send_message(msg)
     smtp.quit()
 
 
@@ -198,18 +199,17 @@ def notify_ignored(config, mail_id, mail, from_):
     msg["To"] = from_
     msg["Bcc"] = recipient
 
-    main_text = MIMEText(IGNORED_MAIL.format(client_id, login, mail_id, from_, config['mailinfos']['sender-pattern']),
-                         "plain")
+#    main_text = MIMEText(IGNORED_MAIL.format(client_id, login, mail_id, from_, config['mailinfos']['sender-pattern']),
+    main_text = MIMEText(IGNORED_MAIL.format(client_id, login, mail_id, from_), "plain")
     msg.attach(main_text)
 
     attachment = MIMEBase("message", "rfc822")
-    attachment.set_payload(mail.as_string())
+    attachment.set_payload(mail.as_string(), 'utf8')
     attachment.add_header("Content-Disposition", "inline")
     msg.attach(attachment)
 
     smtp = SMTP(str(smtp_infos["host"]), int(smtp_infos["port"]))
-    msg_content = msg.as_string().encode("utf8") if six.PY3 else msg.as_string()
-    smtp.sendmail(sender, recipient, msg_content)
+    smtp.send_message(msg)
     smtp.quit()
 
 
@@ -229,8 +229,7 @@ def notify_result(config, subject, message):
     msg.attach(main_text)
 
     smtp = SMTP(str(smtp_infos["host"]), int(smtp_infos["port"]))
-    msg_content = msg.as_string().encode("utf8") if six.PY3 else msg.as_string()
-    smtp.sendmail(sender, recipient, msg_content)
+    smtp.send_message(msg)
     smtp.quit()
 
 
@@ -257,15 +256,51 @@ def get_preview_pdf_path(config, mail_id):
     return os.path.join(output_dir, filename)
 
 
+def modify_attachments(mail_id, attachments):
+    """Remove inline attachments and educe size attachments"""
+    new_lst = []
+    for dic in attachments:
+        # we pass inline image, often used in signature. This image will be in generated pdf
+        if dic['type'].startswith('image/') and dic['disp'] == 'inline':
+            if dev_mode:
+                logger.info("{}: skipped inline image '{}' of size {}".format(mail_id, dic['filename'], dic['len']))
+            continue
+        if dic['type'].startswith('image/') and dic['len'] > 100000:
+            try:
+                img = Image.open(BytesIO(dic['content']))
+            except UnidentifiedImageError as msg:
+                new_lst.append(dic)  # kept original image
+                continue
+            is_reduced, new_size = get_reduced_size(img.size, img_size_limit)
+            new_img = img
+            if is_reduced:
+                if dev_mode:
+                    logger.info("{}: resized image '{}'".format(mail_id, dic['filename']))
+                # see https://pillow.readthedocs.io/en/stable/handbook/concepts.html#filters
+                new_img = img.resize(new_size, Image.BICUBIC)
+
+            new_bytes = BytesIO()
+            # save the image in new_bytes
+            try:
+                new_img.save(new_bytes, format=img.format, optimize=True, quality=75)
+            except ValueError as err:
+                new_img.save(new_bytes, format=img.format, optimize=True)
+            new_content = new_bytes.getvalue()
+            new_len = len(new_content)
+            if new_len < dic['len'] and float(new_len / dic['len']) < 0.9:  # more than 10% of difference
+                dic['filename'] = re.sub(r'(\.[\w]+)$', r'-(redimensionné)\1', dic['filename'])
+                if dev_mode:
+                    logger.info("{}: reduced image '{}' ({} => {})".format(mail_id, dic['filename'], dic['len'],
+                                                                           new_len))
+                dic['len'] = new_len
+                dic['content'] = new_content
+        new_lst.append(dic)
+    return new_lst
+
+
 def send_to_ws(config, headers, main_file_path, attachments, mail_id):
     ws = config["webservice"]
-    client_id = "{0}Z{1}".format(ws['client_id'][:2], ws['client_id'][-4:])
-    counter_dir = Path(ws['counter_dir'])
-    next_id_path = counter_dir / client_id
-    if next_id_path.exists() and next_id_path.read_text():
-        next_id = int(next_id_path.read_text()) + 1
-    else:
-        next_id = 1
+    next_id, client_id = get_next_id(config, dev_infos)
     external_id = "{0}{1:08d}".format(client_id, next_id)
 
     tar_path = Path('/tmp') / '{}.tar'.format(external_id)
@@ -290,50 +325,58 @@ def send_to_ws(config, headers, main_file_path, attachments, mail_id):
             attachment_info.size = len(attachment_contents)
             tar.addfile(tarinfo=attachment_info, fileobj=BytesIO(attachment_contents))
 
-    tar_content = tar_path.read_bytes()
-    now = datetime.now()
-    metadata = {
-        "external_id": external_id,
-        "client_id": client_id,
-        "scan_date": now.strftime("%Y-%m-%d"),
-        "scan_hour": now.strftime("%H:%M:%S"),
-        "user": "testuser",
-        "pc": "pc-scan01",
-        "creator": "scanner",
-        "filesize": len(tar_content),
-        "filename": tar_path.name,
-        "filemd5": md5(tar_content).hexdigest(),
-    }
+    if not dev_mode:  # we send to the ws
+        tar_content = tar_path.read_bytes()
+        now = datetime.now()
+        metadata = {
+            "external_id": external_id,
+            "client_id": client_id,
+            "scan_date": now.strftime("%Y-%m-%d"),
+            "scan_hour": now.strftime("%H:%M:%S"),
+            "user": "testuser",
+            "pc": "pc-scan01",
+            "creator": "scanner",
+            "filesize": len(tar_content),
+            "filename": tar_path.name,
+            "filemd5": md5(tar_content).hexdigest(),
+        }
 
-    auth = (ws['login'], ws['pass'])
-    proto = ws['port'] == '443' and 'https' or 'http'
-    metadata_url = '{proto}://{ws[host]}:{ws[port]}/dms_metadata/{client_id}/{ws[version]}'.format(
-        proto=proto,
-        ws=ws,
-        client_id=client_id,
-    )
-    metadata_req = requests.post(metadata_url, auth=auth, json=metadata)
-    req_content = json.loads(metadata_req.content)
-    if not req_content['success'] or 'id' not in req_content:
-        msg = u"mail_id: {}, code: '{}', error: '{}', metadata: '{}'".format(mail_id, req_content['error_code'],
-                                                                             req_content['error'],
-                                                                             metadata).encode('utf8')
-        raise DmsMetadataError(msg)
-    response_id = req_content['id']
+        auth = (ws['login'], ws['pass'])
+        proto = ws['port'] == '443' and 'https' or 'http'
+        metadata_url = '{proto}://{ws[host]}:{ws[port]}/dms_metadata/{client_id}/{ws[version]}'.format(
+            proto=proto,
+            ws=ws,
+            client_id=client_id,
+        )
+        metadata_req = requests.post(metadata_url, auth=auth, json=metadata)
+        req_content = json.loads(metadata_req.content)
+        # {'message': 'Well done', 'external_id': '05Z507000024176', 'id': 2557054, 'success': True}
+        if not req_content['success'] or 'id' not in req_content:
+            msg = u"mail_id: {}, code: '{}', error: '{}', metadata: '{}'".format(mail_id, req_content['error_code'],
+                                                                                 req_content['error'],
+                                                                                 metadata).encode('utf8')
+            raise DmsMetadataError(msg)
+        response_id = req_content['id']
 
-    upload_url = '{proto}://{ws[host]}:{ws[port]}/file_upload/{ws[version]}/{id}'.format(proto=proto, ws=ws,
-                                                                                         id=response_id)
-    files = {'filedata': ('archive.tar', tar_content, 'application/tar', {'Expires': '0'})}
-    upload_req = requests.post(upload_url, auth=auth, files=files)
-    req_content = json.loads(upload_req.content)
-    if not req_content['success']:
-        msg = u"mail_id: {}, code: '{}', error: '{}'".format(mail_id, req_content['error_code'],
-                                                             req_content.get('error') or
-                                                             req_content['message']).encode('utf8')
-        raise FileUploadError(msg)
+        upload_url = '{proto}://{ws[host]}:{ws[port]}/file_upload/{ws[version]}/{id}'.format(proto=proto, ws=ws,
+                                                                                             id=response_id)
+        files = {'filedata': ('archive.tar', tar_content, 'application/tar', {'Expires': '0'})}
+        ret_content = b''
+        i = 1
+        while ret_content == b'' and i <= 5:  # 5 attempts
+            upload_req = requests.post(upload_url, auth=auth, files=files)
+            ret_content = upload_req.content
+            if i > 1:
+                logger.info("{}: new attempt to upload file".format(mail_id))
+            i += 1
+        req_content = json.loads(ret_content)
+        if not req_content['success']:
+            msg = u"mail_id: {}, code: '{}', error: '{}'".format(mail_id, req_content['error_code'],
+                                                                 req_content.get('error') or
+                                                                 req_content['message']).encode('utf8')
+            raise FileUploadError(msg)
 
-    next_id_txt = str(next_id) if six.PY3 else str(next_id).decode()
-    next_id_path.write_text(next_id_txt)
+        set_next_id(config, next_id)
 
 
 def process_mails():
@@ -364,9 +407,11 @@ def process_mails():
         # handler.mark_reset_ignored('77')
         # res, data = handler.connection.search(None, 'SUBJECT "JBC client"')
         # for mail_id in data[0].split():
-        #      mail = handler.get_mail(mail_id)
-        #      mail_infos = MailData(mail_id, mail)
-        #      email = mail_infos.mail
+        #      omail = handler.get_mail(mail_id)
+        #      parser = Parser(omail, dev_mode, mail_id)
+        #      headers = parser.headers
+        #      amail = parser.message
+        #      parsed = MailParser(omail)
         #     logger.info(email['Subject'])
         handler.disconnect()
         lock.close()
@@ -376,7 +421,7 @@ def process_mails():
         if not mail_id:
             logger.error('Error: you must give an email id (--get_eml=25 by example)')
         mail = handler.get_mail(mail_id)
-        parsed = Parser(mail)
+        parsed = Parser(mail, dev_mode, mail_id)
         logger.info(parsed.headers)
         message = parsed.message
         filename = '{}.eml'.format(mail_id)
@@ -393,11 +438,13 @@ def process_mails():
         if not mail_id:
             logger.error('Error: you must give an email id (--gen_pdf=25 by example)')
         mail = handler.get_mail(mail_id)
-        parsed = Parser(mail)
+        parsed = Parser(mail, dev_mode, mail_id)
         logger.info(parsed.headers)
         pdf_path = get_preview_pdf_path(config, mail_id.encode('utf8'))
         logger.info('Generating {} file'.format(pdf_path))
-        parsed.generate_pdf(pdf_path)
+        payload, cid_parts_used = parsed.generate_pdf(pdf_path)
+        attachments = parsed.attachments(True, cid_parts_used)
+        m_at = modify_attachments(mail_id, attachments)
         handler.disconnect()
         lock.close()
         sys.exit()
@@ -413,42 +460,50 @@ def process_mails():
         sys.exit()
 
     imported = errors = unsupported = ignored = 0
-    # import ipdb; ipdb.set_trace()
     for mail_info in handler.get_waiting_emails():
         mail_id = mail_info.id
         mail = mail_info.mail
         main_file_path = get_preview_pdf_path(config, mail_id)
         try:
-            parser = Parser(mail)
+            parser = Parser(mail, dev_mode, mail_id)
             if parser.origin == 'Generic inbox':
                 mail_sender = parser.headers["From"][0][1]
                 notify_unsupported_origin(config, mail, mail_sender)
-                handler.mark_mail_as_unsupported(mail_id)
+                if not dev_mode:
+                    handler.mark_mail_as_unsupported(mail_id)
                 unsupported += 1
                 continue
             headers = parser.headers
             # we check if the pushing agent has a permitted email format
-            if not check_transferer(headers['Agent'][0][1], config['mailinfos'].get('sender-pattern', '.+')):
-                handler.mark_mail_as_ignored(mail_id)
+            if 'Agent' in headers and not check_transferer(headers['Agent'][0][1],
+                                                           config['mailinfos'].get('sender-pattern', '.+')):
+                if not dev_mode:
+                    handler.mark_mail_as_ignored(mail_id)
                 notify_ignored(config, mail_id, mail, headers['Agent'][0][1])
                 # logger.error('Rejecting {}: {}'.format(headers['Agent'][0][1], headers['Subject']))
                 ignored += 1
                 continue
             # logger.info('Accepting {}: {}'.format(headers['Agent'][0][1], headers['Subject']))
-            attachments = parser.attachments
+            cid_parts_used = set()
             try:
-                parser.generate_pdf(main_file_path)
+                payload, cid_parts_used = parser.generate_pdf(main_file_path)
+                pdf_gen = True
             except Exception as pdf_exc:
                 # if 'XDG_SESSION_TYPE=wayland' not in str(pdf_exc):
                 main_file_path = main_file_path.replace('.pdf', '.eml')
                 save_as_eml(main_file_path, parser.message)
+                pdf_gen = False
+            o_attachments = parser.attachments(pdf_gen, cid_parts_used)
+            attachments = modify_attachments(mail_id, o_attachments)
             send_to_ws(config, headers, main_file_path, attachments, mail_id)
-            handler.mark_mail_as_imported(mail_id)
+            if not dev_mode:
+                handler.mark_mail_as_imported(mail_id)
             imported += 1
         except Exception as e:
             logger.error(e, exc_info=True)
             notify_exception(config, mail_id, mail, e)
-            handler.mark_mail_as_error(mail_id)
+            if not dev_mode:
+                handler.mark_mail_as_error(mail_id)
             errors += 1
 
     logger.info("{} emails have been imported. {} emails are unsupported. {} emails have caused an error. {} emails "
@@ -506,7 +561,7 @@ def clean_mails():
         if not mail:
             error += 1
             continue
-        parser = Parser(mail)
+        parser = Parser(mail, dev_mode, mail_id)
         logger.info(u"{}: '{}'".format(mail_id, parser.headers['Subject']))
         out.append(u"{}: '{}'".format(mail_id, parser.headers['Subject']))
         if doit:
